  #****s* SecDec/general/perlsrc/remakebases.pl
  #  NAME
  #    remakebases.pl
  #
  #  USAGE
  #  is called from polenumerics.pl, subexp.pl, finishnumerics.pl or justnumerics.pl
  #
  #  USED BY 
  #  polenumerics.pl, subexp.pl, finishnumerics.pl, justnumerics.pl
  #  PURPOSE
  #  compiles libbases.a or libcuba.a in a directory where it can then be used to perform the numerical integration.
  #  separate directories can be needed for each pole structure, since the compilation of the executables
  #  may be performed on different machines for different pole structures. This is also true for different calculations
  #  running concurrently. When the input guarantees the program will be run fully locally, the library is made only once, and
  #  shared by the different pole structures.
  #    
  #  INPUTS
  #  parameters parsed via ARGV:
  #  integpath: path where all the source files for numerical integrator can be found
  #  graph: name of graph
  #  polestruct: either the pole structure, or a flag stating that the program is running locally
  #  compiler: compiler to compile library with
  #  makefile: makefile used to compile library
  #    
  #  RESULT
  #  numerical integrator is compiled in a directory ready to be used for numerical integration
  #    
  #  SEE ALSO
  #  polenumerics.pl, subexp.pl, finishnumerics.pl, justnumerics.pl
  #   
  #****
  #
$integpath=$ARGV[0];
$graph=$ARGV[1];
$polestruct=$ARGV[2];
$compiler=$ARGV[3];
$makefile=$ARGV[4];
$integrator=$ARGV[5];
if ($polestruct eq "none"){
 $mfpath="$integpath/$graph"
} else {
 $mfpath="$integpath/$graph/$polestruct"
}
$mftest="$mfpath/$makefile";
if($integrator){$lib="libcuba.a"}else{$lib="libbases.a"};
if (-e $mftest){
$rc=0;
 open (MFT,"<",$mftest);
 while (<MFT>){
  if ($_=~/^FC(.+)=(.+)$compiler/){$rc=1}
 }
 close MFT;
 if($rc==1){
  if(-e "$mfpath/$lib"){
  $currentcomputer=`uname -a`;
  chomp $currentcomputer;
  open (OLDLOG,"<","$mfpath/compile.log");
  @oldinfo=<OLDLOG>;
  close OLDLOG;
  $oldin=$oldinfo[0];
  chomp $oldin;
  if($oldin eq $currentcomputer){exit}
  }
 }
}

if($integrator){
 print "Compiling Cuba library...\n"
}else{
 print "Compiling BASES...\n"
};
makedir("$integpath/$graph");
@intfiles=glob "$integpath/*";
@copfiles=();
foreach $file (@intfiles){
 chomp $file;
 unless(-d $file){unless($file=~m/\.o/){unless($file=~m/\.a/){unless($file=~m/\.svn/){push(@copfiles,$file)}}}}
}
if ($polestruct eq "none") {
 foreach $file (@copfiles){
  system("cp $file $integpath/$graph")
 }
 if($integrator){
  $nsrc="$integpath/$graph/src";
  copycubafiles($nsrc);
 }
 system("cd $integpath/$graph; perl -pi -e 's/FC(.+)=(.+)/FC          = $compiler/g' $makefile");
 system("cd $integpath/$graph;make -s -f $makefile>quiet");
} else {
 makedir("$integpath/$graph");
 if (-d "$integpath/$graph/$polestruct") {
  system("rm $integpath/$graph/$polestruct/*");
 } else { 
  system("mkdir $integpath/$graph/$polestruct");
 }
 foreach $file (@copfiles){
  system("cp $file $integpath/$graph/$polestruct")
 }
 if($integrator){
  $nsrc="$integpath/$graph/$polestruct/src";
  copycubafiles($nsrc);
 } 
 system("cd $integpath/$graph/$polestruct; perl -pi -e 's/FC(.+)=(.+)/FC          = $compiler/g' $makefile");
 system("cd $integpath/$graph/$polestruct;make -s -f $makefile>quiet");
}
system("uname -a >$mfpath/compile.log");

sub makedir{
 foreach $dir (@_){
  unless(-d $dir){system("mkdir $dir")}
 } 
}


sub copycubafiles{
 my $src=$_[0];
 makedir("$src","$src/common","$src/cuhre","$src/divonne","$src/vegas","$src/suave");
 @comfiles=("ChiSquare.c","Erf.c","Random.c","stddecl.h");
 foreach $file (@comfiles){
  system("cp $integpath/src/common/$file $src/common/$file")
 }
 @cuhrefiles=("common.c","Cuhre.c","Cuhre.tm","decl.h","Integrate.c","Rule.c");
 foreach $file (@cuhrefiles){
  system("cp $integpath/src/cuhre/$file $src/cuhre/$file")
 }
 @divonnefiles=("common.c","decl.h","Divonne.c","Divonne.tm","Explore.c","FindMinimum.c","Integrate.c","KorobovCoeff.c","Rule.c","Sample.c","Split.c");
 foreach $file (@divonnefiles){
  system("cp $integpath/src/divonne/$file $src/divonne/$file")
 }
 @vegasfiles=("common.c","decl.h","Grid.c","Integrate.c","Vegas.c","Vegas.tm");
 foreach $file (@vegasfiles){
  system("cp $integpath/src/vegas/$file $src/vegas/$file")
 }
 @suavefiles=("common.c","decl.h","Fluct.c","Grid.c","Integrate.c","Sample.c","Suave.c","Suave.tm");
 foreach $file (@suavefiles){
  system("cp $integpath/src/suave/$file $src/suave/$file")
 }
}
