(*
  #****m* SecDec/loop/src/subexp/formindlist.m
  #  NAME
  #    formindlist.m
  #
  #  USAGE
  #  is loaded by subandexpand*l*h*.m
  # 
  #  USES 
  #  output from the sector decomposition *secP*l*h*.out
  #  routines loaded from Degeneracy.m
  #
  #  USED BY 
  #    
  #  subandexpand*l*h*.m
  #
  #  PURPOSE
  #  takes output from sector decomposition, manipulates it using various symmetries of the problem
  #  to form 'integrandfunctionlist', and also ustore,fstore,nstore,degen, which are then used as the
  #  input for symbsub.m and formfortran.m 
  #    
  #  INPUTS
  #  
  #  from subandexpand*l*h*.m:
  #  path, srcdir: where to find Degeneracy.m
  #  n: number of propagators
  #  logi, lini, higheri: the number of logarithmic, linear and higher order poles respectively
  #  sec*P*l*h*: output from the decomposition, read by subandexpand*l*h*.m, of the form
  #  
  #  variables: 
  #  dieflag: indicates to subandexpand*l*h*.m whether further action is to be taken (ie whether there is any input
  #   from the sector decomposition of this particular pole structure.
  #    
  #  RESULT
  #  integrandfunctionlist is formed, as are ustore, fstore, nstore and degen, for use by formfortran.m and symbsub.m  
  #  
  #  SEE ALSO
  #  subandexpand*l*h*.m, symbsub.m, formfortran.m, Degeneracy.m
  #   
  #****
  *)
  (*populates the list 'plist', which stores info in fstore,ustore,nstore*)
(*ready for symbolic subtraction/eps expansion*)
myclear[varname_]:=Clear[varname];
refexps[expl_,secnum_]:=Module[{newexp},
 newexp=expl;
 newexp[[secnum]]=newexp[[n]];
 newexp=Table[{(newexp[[tabi]])/.eps->0,D[newexp[[tabi]],eps],z[tabi]},{tabi,n-1}]];

ttoz[expr_]:=expr/.t[a_]->z[a];
zntozrnum[expr_,zznum_]:=expr/.z[n]->z[zznum];

reformsubsec[rsec_,rnum_]:=Module[{newsec},
 newsec=ttoz/@rsec;
 newsec=Function[xx,zntozrnum[xx,rnum]]/@newsec;
 newsec[[1]]=refexps[newsec[[1]],rnum];
 newsec];
 
Print["Recasting input for subtraction"];
recasttime=Timing[
 If[
  NumberQ[indsec]
 ,
  listname=StringJoin["sec",ToString[indsec],"P",polestring];
  plist=Symbol[listname];
  myclear[listname];
  plist=Function[xx,reformsubsec[xx,indsec]]/@plist;
  Clear[plistin]
 , 
  Do[
   listname=StringJoin["sec",ToString[seci],"P",polestring];
   plistin[seci]=Symbol[listname];
   myclear[listname];
   plistin[seci]=Function[xx,reformsubsec[xx,seci]]/@plistin[seci]
  ,
   {seci,n}];

  listjoin[{elements1___},{elements2___}]:={elements1,elements2};
  plistfold=Table[plistin[seci],{seci,2,n}];
  plist=Fold[listjoin,plistin[1],plistfold];
  Clear[plistin]];
 If[Or[MatchQ[Head[plist],listjoin],plist=={}],dieflag=1,dieflag=0];][[1]];
Print["Recasting done, time taken = ",recasttime," seconds"];
Print["Integrand takes up ",pbytes=ByteCount[plist]," bytes"]; 
If[
 dieflag==0
,
 ordersector[osec_]:=Module[{osect},
  osect=osec;
  osexp=osect[[1]];
  osexp=Reverse[Sort[osexp]];
  osect[[1]]=osexp;
  osecreps=Table[osexp[[tvar,3]]->z[tvar],{tvar,n-1}];
  osect/.osecreps];
 
 orderlist[oplist_]:=Module[{ifl},ifl=oplist;ordersector/@ifl];

 ordertime=Timing[plist=orderlist[plist];][[1]];
 Print["Integrand list ordered, time taken = ",ordertime," seconds"];
 populatefudegn[popsec_]:=Module[{},
   fudegn++;
   nstore[iflcount,fudegn]=popsec[[4]];
   fstore[iflcount,fudegn]=popsec[[3]];
   ustore[iflcount,fudegn]=popsec[[2]];
   If[
    nstore[iflcount,fudegn]//NumberQ
   ,
    degen[iflcount,fudegn]=nstore[iflcount,fudegn];
    nstore[iflcount,fudegn]=1
   ,
    degen[iflcount,fudegn]=1];];
 
 newifl[nfl_]:=Module[{},iflcount++;fudegn=0;populatefudegn/@nfl;{nfl[[1,1]],fudegn}];
 
 collatelist[cplist_]:=Module[{integrandfunctionlist},
  integrandfunctionlist=Sort[cplist];
  tempintegrandfunctionlist=Split[integrandfunctionlist, #1[[1]] == #2[[1]]&];
  iflcount=0;
  integrandfunctionlist=newifl/@tempintegrandfunctionlist];

 Print["Collating integrand list"];
 collatetime=Timing[plist=collatelist[plist];][[1]];
 Print["Integrand list collated, time taken = ",collatetime,"seconds"];

 Print["Checking for symmetry amongst subsectors"];
 Get[StringJoin[path,srcdir,"/Degeneracy.m"]];	
 onlyz[expr_]:=(expr/.{Times[z[a_],z[b_]]->0,Power[z[c_],d_]->0});

 findsymmetries:=Module[{},
  ifllen=Length[plist];
  Do[
   tempelem=plist[[dovar]];
   storelen=tempelem[[2]];
   degenexplist=tempelem[[1]];
   Clear[tempelem];
   me=matchexps[degenexplist];
   If[
    me=!={}
   ,
    allowedperms=makefullperm[me]
   ,
    allowedperms={identity->identity}
   ];
   Do[
    If[
     degen[dovar,i]=!=0
    ,
     ftest1=fstore[dovar,i]//only2z;
     utest1=ustore[dovar,i]//only2z;
     Do[
      If[
       degen[dovar,j]=!=0
      ,
       ftest2=fstore[dovar,j]//only2z;
       utest2=ustore[dovar,j]//only2z;
       donethistime=False;
							(Or[
							 If[And[MatchQ[utest2/.#,utest1],MatchQ[ftest2/.#,ftest1],MatchQ[nstore[dovar,j]/.#,nstore[dovar,i]]
							  ,MatchQ[ustore[dovar,j]/.#,ustore[dovar,i]],MatchQ[fstore[dovar,j]/.#,fstore[dovar,i]]]
								,
							  degen[dovar,i]=degen[dovar,i]+degen[dovar,j];
         degen[dovar,j]=0;
         fstore[dovar,j]=0;
         ustore[dovar,j]=0;
         nstore[dovar,j]=0;
         True
								,
								 False
	       ]
							])&@@allowedperms
						]
     ,{j,i+1,storelen}]]
   ,{i,storelen-1}]
  ,{dovar,ifllen}];];
 If[
  pbytes<20000000
 ,  
  degentime=Timing[findsymmetries;][[1]];
  Print["Symmetries accounted for, time taken = ",degentime," seconds"];
 ,
  Print["Symmetries not considered"];
  degentime=0
 ];
 integrandfunctionlist=plist;
 Clear[plist];
];
