(*
  #****m* SecDec/loop/src/subexp/formC.m
  #  NAME
  #    formC.m
  #
  #  USAGE
  #  is called by subandexpand*l*h*.m to complete the epsilon expansion, and write the C files f*.c
  #  into the appropriate subdirectories 
  # 
  #  USES 
  #  parts.m, ExpOpt.m, 
  #
  #  USED BY 
  #    
  #  subandexpand*l*h*.m
  #
  #  PURPOSE
  #  writes the C files f*.c in the appropriate subdirectories
  #    
  #  INPUTS
  #  from subandexpand*l*h*:
  #  n: number of propagators
  #  expu, expf: exponents of U and F^-1 respectively
  #  path, srcdir: where to load parts.m, ExpOpt.m from
  #  logi, lini, higheri: the number of logarithmic, linear and higher order poles respectively
  #
  #  originally from formindlist.m:
  #  integrandfunctionlist: contains the list of exponents of each variable, together with the number of functions
  #  with the identical exponent structure
  #  fstore[*,*],ustore[*,*],nstore[*,*],degen[*,*]: the numerator, and functions f and u from each subsector (after
  #  decomposition and permutation of variables to exploit symmetries of the problem), together with any information on
  #  degeneracies of these functions (eg if subsector A == subsector B upto a permutations of variables)
  #   
  #  originally from symbsub.m:
  #  epspower[*]:the power of epsilon as a prefactor in piece * of the subtraction
  #  numcoeff[*]:the O(1) prefactor of the piece * of the subtraction
  #  set[*]: if {x,a} were an element of set[*], this indicates that the piece * of the subtraction is to 
  #   be differentiated 'a' times wrt x, and x is then to be set to zero
  #  exponents[*,**]: the exponent of variable z[**] in piece * of the subtraction
  #
  #
  #  RESULT
  #  C functions f*.c are written in the appropriate subdirectory corresponding to the given graph, pole structure,
  #   order in epsilon, and, when IBP is used, the number of independent variables in f*.c
  #  SEE ALSO
  #  subandexpand*l*h*.m, formindlist.m, symbsub.m, parts.m, ExpOpt.m
  #   
  #****
  *)
  
 (* changes wrt formfortran.m: 
 formnumericalintegrand -> formnumericalintegrandC
 lines 131,193: ToString[functioncounter[epspole]]<>".c"
 lines 137,198,267: writeopt -> writeoptC
 
 *) 
  
numberab[ni_]:=Module[{exps},
exps=integrandfunctionlist[[ni,1]];
Do[a[numberi]=exps[[numberi,1]];b[numberi]=exps[[numberi,2]],{numberi,n-1}];
];
funset[fsj_,fsk_,fsset_]:=
(Fold[der,(fstore[fsj,fsk]^(-expf)ustore[fsj,fsk]^(expu)nstore[fsj,fsk]),fsset]);



der[derfun_,derset_]:=
(D[derfun,derset])/.derset[[1]]->0;


epsexpand[exprtoex_,ordertoex_]:=Table[
 ((D[exprtoex,{eps,epsdif}])/.eps->0)/epsdif!
,
 {epsdif,0,ordertoex}
];

epsmulti[l1_,l2_,ordtom_]:=Table[
 Table[
  l1[[i]]l2[[j-i+1]]
 ,
  {i,j}
 ]
,
 {j,ordtom+1}
];

addelement[ele_]:=Module[{},
 integrands[temppow]={integrands[temppow],ele};
 temppow++;
];
 

populateintlists[poplist_,poppow_]:=Module[{},
 temppow=poppow;
 addelement/@poplist;
];


formnumericalintegrandC[explab_,epsordreq_,0]:=Module[{},
 numberab[explab];
 Clear[integrands];
 Do[integrands[inti]={},{inti,-minpole,epsordreq}];
 Do[
  If[
   epspower[mu]<=epsordreq
  ,
   mufunct=Table[
    If[MatchQ[degen[explab,functionnumber],0],
    0,degen[explab,functionnumber]*funset[explab,functionnumber,set[mu]]]
   ,{functionnumber,integrandfunctionlist[[explab,2]]}];
   mufunct=mufunct//.List[a___,0,b___]->List[a,b];
   expnumfact=Product[z[expi]^exponents[mu,expi],{expi,n-1}]numcoeff[mu];
   exorder=epsordreq-epspower[mu];
   expnfeps=epsexpand[expnumfact,exorder];
   mufunct=Function[xx,epsexpand[xx,exorder]]/@mufunct;
   mufunct=Function[xx,epsmulti[expnfeps,xx,exorder]]/@mufunct;
   Function[xx,populateintlists[xx,epspower[mu]]]/@mufunct;	
  ]
 ,{mu,sizemu}];
 Do[
  If[
   MatchQ[integrands[epsord],{}]==False
  ,
   functiontooptimize=Plus@@Flatten[integrands[epsord]];
   If[
    MatchQ[functiontooptimize,0]==False
   ,
    functioncounter[epsord]++;
    direy=StringJoin[direyt,"/epstothe",ToString[epsord]];
    If[FileNames[direy]=={},CreateDirectory[direy]];
    outfile=StringJoin[direy,"/f",ToString[functioncounter[epsord]],".c"];
    outfile2=StringJoin[direy,"/f",ToString[functioncounter[epsord]],"_fu.c"];
    funcname=StringJoin["P",polestring,"f",ToString[functioncounter[epsord]]];
    funcname2=StringJoin["P",polestring,"f",ToString[functioncounter[epsord]],"="];
    try=OptimizeExpression[functiontooptimize, OptimizationSymbol -> y];
    tt =try/.{OptimizedExpression->Hold};
    he1=tt/.{Block->MyBlock,CompoundExpression->List,Set->Rule};
    expr=ReleaseHold[he1];
    writeoptC[expr,varletter,funcname,outfile];
    (*WriteString[outfile2,funcname2];
				WriteString[outfile2,CAssign[functiontooptimize,AssignOptimize->False]];*)
   ]
  ]
 ,
  {epsord,-minpole,epsordreq}
 ];
 Clear[a,b];
];

Get[StringJoin[path,"src/subexp/","parts.m"]];



formnumericalintegrandC[explab_,epsordreq_,1]:=Module[{},
 numberab[explab];
 Clear[integrands,memcount];
 Do[Do[integrands[epsfort,epspole]={};memcount[epsfort,epspole]=0,{epspole,epsfort,epsordreq}],{epsfort,-minpole,Min[epsordreq,0]}];
 Do[Do[If[epspower[mu]<=epsordreq,
    mufunctpre=1;
    varsset={};
    Do[
     expt=exponents[mu,mui]/.eps->0;
     exptifl=integrandfunctionlist[[explab,1,mui,1]];
     expdif=expt-exptifl;
     If[
      expt <0
     ,
      mufunctpre=mufunctpre*z[mui]^expdif;
      inpa=integrandfunctionlist[[explab,1,mui,2]];
      varsset=Append[varsset,{z[mui],inpa,exptifl}];
     ,
      mufunctpre=mufunctpre*z[mui]^(exponents[mu,mui])
     ];
    ,{mui,n-1}
    ];
    mufunct=mufunctpre*degen[explab,functionnumber]*funset[explab,functionnumber,set[mu]];
    ftoC[mufunct,epspower[mu],numcoeff[mu]];
   ]
  ,{mu,sizemu}
  ];
  Do[
   Do[
    If[
     Or[MatchQ[functionnumber,integrandfunctionlist[[explab,2]]],memcount[epsfort,epspole]>MEMCUTOFF/2]
    ,
     functiontooptimize=Plus@@(Flatten[integrands[epsfort,epspole]]);
     memcount[epsfort,epspole]=0;
     integrands[epsfort,epspole]={};
     If[
      MatchQ[functiontooptimize,0]==False
     ,
      functioncounter[epspole]++;
      direy=StringJoin[direyt,"/epstothe",ToString[epspole]];
      If[FileNames[direy]=={},CreateDirectory[direy]];
      outfile=direy<>"/f"<>ToString[functioncounter[epspole]]<>".c";
      funcname="P"<>polestring<>"f"<>ToString[functioncounter[epspole]];
      outfile2=direy<>"/f"<>ToString[functioncounter[epspole]]<>"_fu.c";
      funcname2=StringJoin["P",polestring,"f",ToString[functioncounter[epspole]],"="];
       try=OptimizeExpression[functiontooptimize, OptimizationSymbol -> y];
      tt =try/.{OptimizedExpression->Hold};
      he1=tt/.{Block->MyBlock,CompoundExpression->List,Set->Rule};
      expr=ReleaseHold[he1];
      writeoptC[expr,varletter,funcname,outfile];
(*      WriteString[outfile2,funcname2];
						WriteString[outfile2,CAssign[functiontooptimize,AssignOptimize->False]];
*)
      (*Save[outfile<>".m",functiontooptimize]*)
     ]
    ]
   ,{epspole,epsfort,epsordreq}
   ]
  ,{epsfort,-minpole,Min[epsordreq,0]}
  ]
 ,{functionnumber,integrandfunctionlist[[explab,2]]}
 ];
 Clear[a,b];
];


EXPLICITIZE[Fmf_,Eepsfort_]:=Module[{postDmf,TIMESmf},
 Clear[explicitmf,explicitTIMES];
 explicitmf[emfa__]:=0;
 lenemfa[emfa_]:=0;
 If[ListQ[Fmf],
  postDmf=Fmf[[1]];
  TIMESmf=Times@@Fmf[[2]];
  Do[
   explicitmf[emfa]=postepsD[postDmf,{eps,emfa}];
   explicitmf[emfa]=explicitmf[emfa]/.eps->0;
   If[MatchQ[Head[explicitmf[emfa]],List]
   ,
    explicitmf[emfa]=Flatten[explicitmf[emfa]];
    lenemfa[emfa]=Length[explicitmf[emfa]];
    Do[explicitmf[emfa,emfb]=((explicitmf[emfa][[emfb]])/.postD->D)/emfa!
    ,{emfb,lenemfa[emfa]}]
   ,explicitmf[emfa,1]=((explicitmf[emfa])/.postD->D)/emfa!;
    lenemfa[emfa]=1;
   ];
   explicitmf[emfa]=0;
   explicitTIMES[emfa]=((D[TIMESmf,{eps,emfa}])/.eps->0)/emfa!;
  ,{emfa,0,thisepsorder}]
 ,TIMESmf=Fmf;
  Do[
   explicitTIMES[emfa]=((D[TIMESmf,{eps,emfa}])/.eps->0)/emfa!
  ,{emfa,0,thisepsorder}];
  explicitmf[0,1]=1;lenemfa[0]=1;
 ];
 postDpart[pDa_]:=Table[postDpart[pDa,pDi],{pDi,lenemfa[pDa]}];
 thisepsexpandedfunct=Flatten/@epsexpandedfunct;
 currentepsorder=Eepsfort;
 INTEGRANDIZE/@thisepsexpandedfunct;
 Clear[explicitmf,explicitTIMES,postDmf,TIMESmf];
];

INTEGRANDIZE[intlist_]:=Module[{nextelement,nextmemcount},
 If[ListQ[intlist],INTEGRANDIZE/@intlist;currentepsorder++,
  nextelement=intlist/.{TIMESpart[Tpa_]->explicitTIMES[Tpa]};
  nextelement=nextelement/.postDpart->explicitmf;
  nextmemcount=ByteCount[nextelement];
  If[And[MatchQ[integrands[epspower[mu],currentepsorder],{}]==False,memcount[epspower[mu],currentepsorder]+nextmemcount>MEMCUTOFF]
  ,
   Print["Threshold reached, eps order = ",currentepsorder,", memcount = ",memcount[epspower[mu],currentepsorder]];
   functiontooptimize=Plus@@(Flatten[integrands[epspower[mu],currentepsorder]]);
   If[MatchQ[functiontooptimize,0]==False,
    integrands[epspower[mu],currentepsorder]={};memcount[epspower[mu],currentepsorder]=0;
    functioncounter[currentepsorder]++;
    direy=StringJoin[direyt,"/epstothe",ToString[currentepsorder]];
    If[FileNames[direy]=={},CreateDirectory[direy]];
    outfile=StringJoin[direy,"/f",ToString[functioncounter[currentepsorder]],".c"];
    outfile2=direy<>"/f"<>ToString[functioncounter[currentepsorder]]<>"_fu.c";
    funcname=StringJoin["P",polestring,"f",ToString[functioncounter[currentepsorder]]];
    funcname2=StringJoin["P",polestring,"f",ToString[functioncounter[currentepsorder]],"="];
    try=OptimizeExpression[functiontooptimize, OptimizationSymbol -> y];
    tt =try/.{OptimizedExpression->Hold};
    he1=tt/.{Block->MyBlock,CompoundExpression->List,Set->Rule};
    expr=ReleaseHold[he1];
    writeoptC[expr,varletter,funcname,outfile];
(*    WriteString[outfile2,funcname2];
   	WriteString[outfile2,CAssign[functiontooptimize,AssignOptimize->False]];
*)
   ]
  ];
  integrands[epspower[mu],currentepsorder]={integrands[epspower[mu],currentepsorder],nextelement};
  memcount[epspower[mu],currentepsorder]=memcount[epspower[mu],currentepsorder]+nextmemcount
 ]
];



ftoC[ffort_,epsfort_,nfort_]:=Module[{},
 mf=Flatten[intparts[ffort,varsset,precisionrequired-epsfort]];
 mf=mf//.{A___,0,B___}->{A,B};
 If[
  MatchQ[mf,{}]==False
 ,
  mf=((#/.ipsum->ipexplicitsum)*nfort)&/@mf;
  mf=mf/.postD->postDtemp;
  mf=mf/.Times[TB___,postDtemp[pDf__],TA___]->{postDtemp[pDf],TIMES[TB,TA]};
  mf=mf/.postDtemp->postD;
  thisepsorder=precisionrequired-epsfort;
  Clear[postDpart];
  postDlist=Table[postDpart[pDpi],{pDpi,0,thisepsorder}];
  TIMESlist=Table[TIMESpart[tpi],{tpi,0,thisepsorder}];
  epsexpandedfunct=epsmulti[postDlist,TIMESlist,thisepsorder];
  EXPLICITIZE[#,epsfort]&/@mf;
 ]
];
		
direy=StringJoin[outp,"/",polestring]; (*directory for the output files*)
direycount=0;
direyt=direy;
While[
	FileNames[direyt]=!={}
,
	direycount++;
	direyt=StringJoin[direy,ToString[direycount]]
];
If[direycount>0,RenameDirectory[direy,direyt]];(*puts old results into another directory, ~diagramname#, where the
largest # relates to the most recent folder*)
CreateDirectory[direy]; (*Creates the directory to save the files to. Most recent directory is ~diagramname*)
direyt=direy;
minpole = n-jexp;
varletter="y";
MyBlock[listvar_,listabbr_]:={listvar,listabbr};
Clear[x];
Do[
	z[changezi]=ToExpression[StringJoin["x",ToString[changezi-1]]]
,
	{changezi,n-1}
];

invmap[ia_]:=(sp[ia[[1]],ia[[2]]]=es[ia[[3]]]);
invarremap[exl_]:={{1,2,0},{2,3,1},{1,3,2}}/;exl<=4
invarremap[5]={{1,2,0},{2,3,1},{1,3,2},{1,4,3},
		 {2,4,4},{3,4,5},{1,5,6},{2,5,7},{3,5,8},{4,5,9}};
invarremap[exl_]:=Module[{a},
 a=invarremap[exl-1];
	from=Length[a];
	Join[a,Table[{i,exl,i+from-1},{i,exl-1}]]];
(*	
If[extlegs<=4,
 invariantremapping={{1,2,0},{2,3,1},{1,3,2}};
,If[extlegs==5,
	 invariantremapping={{1,2,0},{2,3,1},{1,3,2},{1,4,3},
		 {2,4,4},{3,4,5},{1,5,6},{2,5,7},{3,5,8},{4,5,9}}
	,invariantremapping={{1,2,0},{2,3,1},{1,3,2},{1,4,3},
		 {2,4,4},{3,4,5},{1,5,6},{2,5,7},{3,5,8},{4,5,9},
			{1,6,10},{2,6,11},{3,6,12},{4,6,13},{5,6,14}}]];
*)
invariantremapping=invarremap[extlegs];
invmap/@invariantremapping;(*defines the remapping sp[i,j]->es[k] depending on #exernal legs*)
ms[i_]:=em[i-1]; (*defines the remapping of ms*)
ssp[i_]:=esx[i-1];(*and ssp*)
xlambda=xvals[0];xalpha=xvals[1];xbeta=xvals[2];
(*csrp[ss_,si_]:=ss<>"("<>si<>".)"->ss<>"["<>si<>"]";
csrplist[css_,csimin_,csimax_]:=csrp[css,#]&/@Table[ToString[i],{i,csimin,csimax}];
Cstringreplacements=Flatten[{
csrplist["es",0,invariantremapping[[-1,-1]]],
csrplist["em",0,n-1],
csrplist["esx",0,extlegs-1],
csrplist["xvals",0,2]}];*)
(*FORMATREPS=Join[(#<>"("~~ShortestMatch[ar__]~~".)"->#<>"["~~ar~~"]")&/@{varletter,"es","esx","em","xvals"},
 {"1/"<>varletter->"1./"<>varletter}];*)
FORMATREPS=(#<>"("~~ShortestMatch[ar__]~~".)"->#<>"["~~ar~~"]")&/@{varletter,"es","esx","em","xvals"};
(*
Do[ 
  Do[
sp[i,j]=Symbol[StringJoin["es",ToString[i],ToString[j]]];
  ,{j,i+1,6}];
ssp[i]=Symbol[StringJoin["es",ToString[i]]];
ms[i]=Symbol[StringJoin["em",ToString[i]]];
, {i,6}];
*)
(* ************************* change this for realistic C template ************************* *)
(* iwmax should be set to true or realistic value for each integral *) 
createoptimizedC:=Module[{iwmax},
iwmax=12;
Cstring0=";";
(*Cstring1="#include<math.h>\ndouble ";
Cstring2="(double x[]) {\n";
Cstring3="double es12,es23,es13,es14,es24,es34,es15,es25,es35,es45,es16,es26,es36,es46,es56,es1,es2,es3,es4,es5,es6,em1,em2,em3,em4,em5,em6;\n";
Cstring3a=StringJoin["const int ndi=",ToString[n-1],";\n"];
(*Cstring3b="for (i=0;i <= ndi; ++i) {xvector.push x[i]}";*)
Cstring4="double ";*)
Cstring1=StringJoin@@("#include<"<>#<>">\n"&/@{"math.h","complex","iostream"});
Cstring1=Cstring1<>"using namespace std;\n";
Cstring1=Cstring1<>"typedef complex<double> dcmplx;\n";
Cstring1=Cstring1<>"dcmplx ";
Cstring2="(const double x[],double es[],double esx[],double em[],double xvals[]) {\n";
Cstring3=Cstring3a="";
Cstring4=StringJoin@@("double x"<>#<>"=x["<>#<>"];\n"&/@Table[ToString[i],{i,0,n-2}]);
(*Do[
Cstring4 = StringJoin[Cstring4, "x", ToString[dotfs],","];
,{dotfs,n-2}];
Cstring4 = StringJoin[Cstring4, "x", ToString[n-1],";\n"];*)
Cstring4a[co_]:="dcmplx "<>varletter<>"["<>ToString[co]<>"];\n";
(*Do[
Cstring4a = StringJoin[Cstring4a, "w", ToString[iw],","];
,{iw,iwmax-1}];
Cstring4a = StringJoin[Cstring4a, "w", ToString[iwmax],";\n"];
*)
(*tempfortstring = "";
Do[
	tempfortstring = StringJoin[tempfortstring, "x", ToString[dotfs]," = x[", ToString[dotfs],"];\n"]
,{dotfs,n-1}
];
Cstring5=tempfortstring;*)
Cstring5="dcmplx FOUT;\n";
Cstring6="return ";
Cstring7=";}";
Do[functioncounter[tiepspow]=0,{tiepspow,-minpole,precisionrequired}];
Print["Memory in use before optimizing = ",MemoryInUse[]];
Do[
	formnumericalintegrandC[numintegdo,precisionrequired,partsflag];
	If[
		Or[numintegdo<10,Mod[numintegdo,10]==0,numintegdo==Length[integrandfunctionlist]]
	,
		Print["numericalintegrand ", numintegdo, " evaluated, Memory in use = ",MemoryInUse[]]
	];
,
	{numintegdo,Length[integrandfunctionlist]}
];
While[functioncounter[-minpole]==0,minpole--];
infostream = OpenWrite[StringJoin[direyt,"/infofile"]];
Do[
	Write[infostream, StringJoin[ToString[tiepspow],"functions = ",
	ToString[functioncounter[tiepspow]]]]
,
	{tiepspow,-minpole,precisionrequired}
];
Close[infostream];
];

(* ******************************************************************************************** *)

Print["Producing C functions"];
forttime=Timing[createoptimizedC;][[1]];
Print["C functions produced, time taken = ",forttime," seconds"];
