(*
  #****m* SecDec/loop/src/subexp/formfortran.m
  #  NAME
  #    formfortran.m
  #
  #  USAGE
  #  is called by subandexpand*l*h*.m to complete the epsilon expansion, and write the fortran files f*.f
  #  into the appropriate subdirectories 
  # 
  #  USES 
  #  parts.m, ExpOpt.m, 
  #
  #  USED BY 
  #    
  #  subandexpand*l*h*.m
  #
  #  PURPOSE
  #  writes the fortran files f*.f in the appropriate subdirectories
  #    
  #  INPUTS
  #  from subandexpand*l*h*:
  #  n: number of propagators
  #  expu, expf: exponents of U and F^-1 respectively
  #  path, srcdir: where to load parts.m, ExpOpt.m from
  #  logi, lini, higheri: the number of logarithmic, linear and higher order poles respectively
  #
  #  originally from formindlist.m:
  #  integrandfunctionlist: contains the list of exponents of each variable, together with the number of functions
  #  with the identical exponent structure
  #  fstore[*,*],ustore[*,*],nstore[*,*],degen[*,*]: the numerator, and functions f and u from each subsector (after
  #  decomposition and permutation of variables to exploit symmetries of the problem), together with any information on
  #  degeneracies of these functions (eg if subsector A == subsector B upto a permutations of variables)
  #   
  #  originally from symbsub.m:
  #  epspower[*]:the power of epsilon as a prefactor in piece * of the subtraction
  #  numcoeff[*]:the O(1) prefactor of the piece * of the subtraction
  #  set[*]: if {x,a} were an element of set[*], this indicates that the piece * of the subtraction is to 
  #   be differentiated 'a' times wrt x, and x is then to be set to zero
  #  exponents[*,**]: the exponent of variable z[**] in piece * of the subtraction
  #
  #
  #  RESULT
  #  fortran functions f*.f are written in the appropriate subdirectory corresponding to the given graph, pole structure,
  #   order in epsilon, and, when IBP is used, the number of independent variables in f*.f
  #  SEE ALSO
  #  subandexpand*l*h*.m, formindlist.m, symbsub.m, parts.m, ExpOpt.m
  #   
  #****
  *)
  
  
  
numberab[ni_]:=Module[{exps},
exps=integrandfunctionlist[[ni,1]];
Do[a[numberi]=exps[[numberi,1]];b[numberi]=exps[[numberi,2]],{numberi,n-1}];
];
funset[fsj_,fsk_,fsset_]:=
(Fold[der,(fstore[fsj,fsk]^(-expf)ustore[fsj,fsk]^(expu)nstore[fsj,fsk]),fsset]);



der[derfun_,derset_]:=
(D[derfun,derset])/.derset[[1]]->0;


epsexpand[exprtoex_,ordertoex_]:=Table[
 ((D[exprtoex,{eps,epsdif}])/.eps->0)/epsdif!
,
 {epsdif,0,ordertoex}
];

epsmulti[l1_,l2_,ordtom_]:=Table[
 Table[
  l1[[i]]l2[[j-i+1]]
 ,
  {i,j}
 ]
,
 {j,ordtom+1}
];

addelement[ele_]:=Module[{},
 integrands[temppow]={integrands[temppow],ele};
 temppow++;
];
 

populateintlists[poplist_,poppow_]:=Module[{},
 temppow=poppow;
 addelement/@poplist;
];


formnumericalintegrand[explab_,epsordreq_,0]:=Module[{},
 numberab[explab];
 Clear[integrands];
 Do[integrands[inti]={},{inti,-minpole,epsordreq}];
 Do[
  If[
   epspower[mu]<=epsordreq
  ,
   mufunct=Table[
    If[MatchQ[degen[explab,functionnumber],0],
    0,degen[explab,functionnumber]*funset[explab,functionnumber,set[mu]]]
   ,{functionnumber,integrandfunctionlist[[explab,2]]}];
   mufunct=mufunct//.List[a___,0,b___]->List[a,b];
   expnumfact=Product[z[expi]^exponents[mu,expi],{expi,n-1}]numcoeff[mu];
   exorder=epsordreq-epspower[mu];
   expnfeps=epsexpand[expnumfact,exorder];
   mufunct=Function[xx,epsexpand[xx,exorder]]/@mufunct;
   mufunct=Function[xx,epsmulti[expnfeps,xx,exorder]]/@mufunct;
   Function[xx,populateintlists[xx,epspower[mu]]]/@mufunct;	
  ]
 ,{mu,sizemu}];
 Do[
  If[
   MatchQ[integrands[epsord],{}]==False
  ,
   functiontooptimize=Plus@@Flatten[integrands[epsord]];
   If[
    MatchQ[functiontooptimize,0]==False
   ,
    functioncounter[epsord]++;
    direy=StringJoin[direyt,"/epstothe",ToString[epsord]];
    If[FileNames[direy]=={},CreateDirectory[direy]];
    outfile=StringJoin[direy,"/f",ToString[functioncounter[epsord]],".f"];
    funcname=StringJoin["P",polestring,"f",ToString[functioncounter[epsord]]];
    try=OptimizeExpression[functiontooptimize, OptimizationSymbol -> w];
    tt =try/.{OptimizedExpression->Hold};
    he1=tt/.{Block->MyBlock,CompoundExpression->List,Set->Rule};
    expr=ReleaseHold[he1];
    writeopt[expr,varletter,funcname,outfile];
    (*Save[outfile<>".m",functiontooptimize]*)
   ]
  ]
 ,
  {epsord,-minpole,epsordreq}
 ];
 Clear[a,b];
];

Get[StringJoin[path,"src/subexp/","parts.m"]];
formnumericalintegrand[explab_,epsordreq_,1]:=Module[{},
 numberab[explab];
 Clear[integrands,memcount];
 Do[Do[integrands[epsfort,epspole]={};memcount[epsfort,epspole]=0,{epspole,epsfort,epsordreq}],{epsfort,-minpole,Min[epsordreq,0]}];
 Do[Do[If[epspower[mu]<=epsordreq,
    mufunctpre=1;
    varsset={};
    Do[
     expt=exponents[mu,mui]/.eps->0;
     exptifl=integrandfunctionlist[[explab,1,mui,1]];
     expdif=expt-exptifl;
     If[
      expt <0
     ,
      mufunctpre=mufunctpre*z[mui]^expdif;
      inpa=integrandfunctionlist[[explab,1,mui,2]];
      varsset=Append[varsset,{z[mui],inpa,exptifl}];
     ,
      mufunctpre=mufunctpre*z[mui]^(exponents[mu,mui])
     ];
    ,{mui,n-1}
    ];
    mufunct=mufunctpre*degen[explab,functionnumber]*funset[explab,functionnumber,set[mu]];
    ftofortran[mufunct,epspower[mu],numcoeff[mu]];
   ]
  ,{mu,sizemu}
  ];
  Do[
   Do[
    If[
     Or[MatchQ[functionnumber,integrandfunctionlist[[explab,2]]],memcount[epsfort,epspole]>MEMCUTOFF/2]
    ,
     functiontooptimize=Plus@@(Flatten[integrands[epsfort,epspole]]);
     memcount[epsfort,epspole]=0;
     integrands[epsfort,epspole]={};
     If[
      MatchQ[functiontooptimize,0]==False
     ,
      functioncounter[epspole]++;
      direy=StringJoin[direyt,"/epstothe",ToString[epspole]];
      If[FileNames[direy]=={},CreateDirectory[direy]];
      outfile=direy<>"/f"<>ToString[functioncounter[epspole]]<>".f";
      funcname="P"<>polestring<>"f"<>ToString[functioncounter[epspole]];
      try=OptimizeExpression[functiontooptimize, OptimizationSymbol -> w];
      tt =try/.{OptimizedExpression->Hold};
      he1=tt/.{Block->MyBlock,CompoundExpression->List,Set->Rule};
      expr=ReleaseHold[he1];
      writeopt[expr,varletter,funcname,outfile];
      (*Save[outfile<>".m",functiontooptimize]*)
     ]
    ]
   ,{epspole,epsfort,epsordreq}
   ]
  ,{epsfort,-minpole,Min[epsordreq,0]}
  ]
 ,{functionnumber,integrandfunctionlist[[explab,2]]}
 ];
 Clear[a,b];
];


EXPLICITIZE[Fmf_,Eepsfort_]:=Module[{postDmf,TIMESmf},
 Clear[explicitmf,explicitTIMES];
 explicitmf[emfa__]:=0;
 lenemfa[emfa_]:=0;
 If[ListQ[Fmf],
  postDmf=Fmf[[1]];
  TIMESmf=Times@@Fmf[[2]];
  Do[
   explicitmf[emfa]=postepsD[postDmf,{eps,emfa}];
   explicitmf[emfa]=explicitmf[emfa]/.eps->0;
   If[MatchQ[Head[explicitmf[emfa]],List]
   ,
    explicitmf[emfa]=Flatten[explicitmf[emfa]];
    lenemfa[emfa]=Length[explicitmf[emfa]];
    Do[explicitmf[emfa,emfb]=((explicitmf[emfa][[emfb]])/.postD->D)/emfa!
    ,{emfb,lenemfa[emfa]}]
   ,explicitmf[emfa,1]=((explicitmf[emfa])/.postD->D)/emfa!;
    lenemfa[emfa]=1;
   ];
   explicitmf[emfa]=0;
   explicitTIMES[emfa]=((D[TIMESmf,{eps,emfa}])/.eps->0)/emfa!;
  ,{emfa,0,thisepsorder}]
 ,TIMESmf=Fmf;
  Do[
   explicitTIMES[emfa]=((D[TIMESmf,{eps,emfa}])/.eps->0)/emfa!
  ,{emfa,0,thisepsorder}];
  explicitmf[0,1]=1;lenemfa[0]=1;
 ];
 postDpart[pDa_]:=Table[postDpart[pDa,pDi],{pDi,lenemfa[pDa]}];
 thisepsexpandedfunct=Flatten/@epsexpandedfunct;
 currentepsorder=Eepsfort;
 INTEGRANDIZE/@thisepsexpandedfunct;
 Clear[explicitmf,explicitTIMES,postDmf,TIMESmf];
];

INTEGRANDIZE[intlist_]:=Module[{nextelement,nextmemcount},
 If[ListQ[intlist],INTEGRANDIZE/@intlist;currentepsorder++,
  nextelement=intlist/.{TIMESpart[Tpa_]->explicitTIMES[Tpa]};
  nextelement=nextelement/.postDpart->explicitmf;
  nextmemcount=ByteCount[nextelement];
  If[And[MatchQ[integrands[epspower[mu],currentepsorder],{}]==False,memcount[epspower[mu],currentepsorder]+nextmemcount>MEMCUTOFF]
  ,
   Print["Threshold reached, eps order = ",currentepsorder,", memcount = ",memcount[epspower[mu],currentepsorder]];
   functiontooptimize=Plus@@(Flatten[integrands[epspower[mu],currentepsorder]]);
   If[MatchQ[functiontooptimize,0]==False,
    integrands[epspower[mu],currentepsorder]={};memcount[epspower[mu],currentepsorder]=0;
    functioncounter[currentepsorder]++;
    direy=StringJoin[direyt,"/epstothe",ToString[currentepsorder]];
    If[FileNames[direy]=={},CreateDirectory[direy]];
    outfile=StringJoin[direy,"/f",ToString[functioncounter[currentepsorder]],".f"];
    funcname=StringJoin["P",polestring,"f",ToString[functioncounter[currentepsorder]]];
     try=OptimizeExpression[functiontooptimize, OptimizationSymbol -> w];
    tt =try/.{OptimizedExpression->Hold};
    he1=tt/.{Block->MyBlock,CompoundExpression->List,Set->Rule};
    expr=ReleaseHold[he1];
    writeopt[expr,varletter,funcname,outfile];
    (*Save[outfile<>".m",functiontooptimize]*)
   ]
  ];
  integrands[epspower[mu],currentepsorder]={integrands[epspower[mu],currentepsorder],nextelement};
  memcount[epspower[mu],currentepsorder]=memcount[epspower[mu],currentepsorder]+nextmemcount
 ]
];



ftofortran[ffort_,epsfort_,nfort_]:=Module[{},
 mf=Flatten[intparts[ffort,varsset,precisionrequired-epsfort]];
 mf=mf//.{A___,0,B___}->{A,B};
 If[
  MatchQ[mf,{}]==False
 ,
  mf=((#/.ipsum->ipexplicitsum)*nfort)&/@mf;
  mf=mf/.postD->postDtemp;
  mf=mf/.Times[TB___,postDtemp[pDf__],TA___]->{postDtemp[pDf],TIMES[TB,TA]};
  mf=mf/.postDtemp->postD;
  thisepsorder=precisionrequired-epsfort;
  Clear[postDpart];
  postDlist=Table[postDpart[pDpi],{pDpi,0,thisepsorder}];
  TIMESlist=Table[TIMESpart[tpi],{tpi,0,thisepsorder}];
  epsexpandedfunct=epsmulti[postDlist,TIMESlist,thisepsorder];
  EXPLICITIZE[#,epsfort]&/@mf;
 ]
];
		
direy=StringJoin[outp,"/",polestring]; (*directory for the output files*)
direycount=0;
direyt=direy;
While[
	FileNames[direyt]=!={}
,
	direycount++;
	direyt=StringJoin[direy,ToString[direycount]]
];
If[direycount>0,RenameDirectory[direy,direyt]];(*puts old results into another directory, ~diagramname#, where the
largest # relates to the most recent folder*)
CreateDirectory[direy]; (*Creates the directory to save the files to. Most recent directory is ~diagramname*)
direyt=direy;
minpole = n-jexp;
varletter="w";
MyBlock[listvar_,listabbr_]:={listvar,listabbr};
Clear[x];
Do[
	z[changezi]=ToExpression[StringJoin["x",ToString[changezi]]]
,
	{changezi,n-1}
];
Do[ 
  Do[
sp[i,j]=Symbol[StringJoin["es",ToString[i],ToString[j]]];
  ,{j,i+1,6}];
ssp[i]=Symbol[StringJoin["es",ToString[i]]];
ms[i]=Symbol[StringJoin["em",ToString[i]]];
, {i,6}];

createoptimizedfortran:=Module[{},
fortranstring1="      double precision function ";
fortranstring0="(x)
      implicit double precision (a-h,o-z)
      integer ndi
      common/mand/es12,es23,es13,es14,es24,es34, 
     & es15,es25,es35,es45,es16,es26,es36,es46,es56
      common/psq/es1,es2,es3,es4,es5,es6
      common/mas/em1,em2,em3,em4,em5,em6
      common/ndimen/ndi
      common/Euler/EulerGamma
      dimension x(ndi)
";
tempfortstring = "";
Do[
	tempfortstring = StringJoin[tempfortstring, "      x", ToString[dotfs]," = x(", ToString[dotfs],")
"]
,
	{dotfs,n-1}
];
fortranstring2 = StringJoin[fortranstring0, tempfortstring];
fortranstring3="
      return
      end";
Do[functioncounter[tiepspow]=0,{tiepspow,-minpole,precisionrequired}];
Print["Memory in use before optimizing = ",MemoryInUse[]];
Do[
	formnumericalintegrand[numintegdo,precisionrequired,partsflag];
	If[
		Or[numintegdo<10,Mod[numintegdo,10]==0,numintegdo==Length[integrandfunctionlist]]
	,
		Print["numericalintegrand ", numintegdo, " evaluated, Memory in use = ",MemoryInUse[]]
	];
,
	{numintegdo,Length[integrandfunctionlist]}
];
While[functioncounter[-minpole]==0,minpole--];
infostream = OpenWrite[StringJoin[direyt,"/infofile"]];
Do[
	Write[infostream, StringJoin[ToString[tiepspow],"functions = ",
	ToString[functioncounter[tiepspow]]]]
,
	{tiepspow,-minpole,precisionrequired}
];
Close[infostream];
];
Print["Producing fortran functions"];
forttime=Timing[createoptimizedfortran;][[1]];
Print["Fortran functions produced, time taken = ",forttime," seconds"];
